"""
A simple example to demonstrate the usage of `WebSearchRAGPipeline`.
"""

import logging
from typing import Optional

from r2r.core import (
    LLMProvider,
    LoggingDatabaseConnection,
    PromptProvider,
    VectorDBProvider,
    log_execution_to_db,
)
from r2r.embeddings import OpenAIEmbeddingProvider
from r2r.integrations import SerperClient
from r2r.pipelines import BasicPromptProvider

from ..basic.rag import BasicRAGPipeline

logger = logging.getLogger(__name__)


class WebSearchRAGPipeline(BasicRAGPipeline):
    def __init__(
        self,
        llm: LLMProvider,
        db: VectorDBProvider,
        embedding_model: str,
        embeddings_provider: OpenAIEmbeddingProvider,
        logging_connection: Optional[LoggingDatabaseConnection] = None,
        prompt_provider: Optional[PromptProvider] = BasicPromptProvider(),
    ) -> None:
        logger.debug(f"Initalizing `WebSearchRAGPipeline`.")
        super().__init__(
            llm=llm,
            logging_connection=logging_connection,
            db=db,
            embedding_model=embedding_model,
            embeddings_provider=embeddings_provider,
            prompt_provider=prompt_provider,
        )
        self.serper_client = SerperClient()

    def transform_query(self, query: str) -> str:
        # Placeholder for query transformation - A unit transformation.
        return query

    @log_execution_to_db
    def search(
        self,
        transformed_query: str,
        filters: dict,
        limit: int,
        *args,
        **kwargs,
    ) -> list:
        results = []
        local_results = super().search(transformed_query, filters, limit)
        results.extend(
            [{"type": "local", "result": ele} for ele in local_results]
        )

        external_results = self.serper_client.get_raw(transformed_query, limit)
        results.extend(
            [{"type": "external", "result": ele} for ele in external_results]
        )

        return results

    @log_execution_to_db
    def construct_context(self, results: list) -> str:
        local_context = super().construct_context(
            [ele["result"] for ele in results if ele["type"] == "local"]
        )
        web_context = self.serper_client.construct_context(
            [ele["result"] for ele in results if ele["type"] == "external"]
        )
        return local_context + "\n\n" + web_context



import asyncio
import os

from r2r.client import R2RClient
from r2r.core.utils import generate_id_from_label

# Initialize the client with the base URL of your API
base_url = "http://localhost:8000"
client = R2RClient(base_url)

from r2r.client import R2RClient

# print('eval_result:', eval_result)
print("Upserting entry to remote db...")
# Upsert a single entry

entry_response = client.add_entry(
    generate_id_from_label("doc 1"),
    {"txt": "This is a test entry"},
    {"tags": ["example", "test"]},
    do_upsert=True,
)
print(f"Upsert entry response:\n{entry_response}\n\n")


# entry_response = client.add_entry(
#     generate_id_from_label("doc 1"),
#     {"txt": "This is a test entry"},
#     {"tags": ["example", "test"]},
#     do_upsert=False,
# )
# print(f"Copy same entry response:\n{entry_response}\n\n")


# print("Upserting entries to remote db...")
# # Upsert multiple entries
# entries = [
#     {
#         "document_id": generate_id_from_label("doc 2"),
#         "blobs": {"txt": "Second test entry"},
#         "metadata": {"tags": "bulk"},
#     },
#     {
#         "document_id": generate_id_from_label("doc 3"),
#         "blobs": {"txt": "Third test entry"},
#         "metadata": {"tags": "example"},
#     },
# ]
# bulk_upsert_response = client.add_entries(entries, do_upsert=True)
# print(f"Upsert entries response:\n{bulk_upsert_response}\n\n")

# # Perform a search
# print("Searching remote db...")
# search_response = client.search("test", 5)
# print(f"Search response:\n{search_response}\n\n")

# print("Searching remote db with filter...")
# # Perform a search w/ filter
# filtered_search_response = client.search("test", 5, filters={"tags": "bulk"})
# print(f"Search response w/ filter:\n{filtered_search_response}\n\n")

# print("Deleting sample document in remote db...")
# # Delete a document
# response = client.filtered_deletion(
#     "document_id", generate_id_from_label("doc 2")
# )
# print(f"Deletion response:\n{response}\n\n")

# print("Searching remote db with filter after deletion...")
# # Perform a search w/ filter after deletion
# post_deletion_filtered_search_response = client.search(
#     "test", 5, filters={"tags": "bulk"}
# )
# print(
#     f"Search response w/ filter+deletion:\n{post_deletion_filtered_search_response}\n\n"
# )

# # Example file path for upload
# # get file directory
# current_file_directory = os.path.dirname(os.path.realpath(__file__))

# file_path = os.path.join(current_file_directory, "..", "data", "test.pdf")

# print(f"Uploading and processing file: {file_path}...")
# # # Upload and process a file
# metadata = {"tags": ["example", "test"]}
# upload_pdf_response = client.upload_and_process_file(
#     generate_id_from_label("pdf 1"), file_path, metadata, None
# )
# print(f"Upload test pdf response:\n{upload_pdf_response}\n\n")

# print("Searching remote db after upload...")
# # Perform a search on this file
# pdf_filtered_search_response = client.search(
#     "what is a cool physics equation?",
#     5,
#     filters={"document_id": generate_id_from_label("pdf 1")},
# )
# print(
#     f"Search response w/ uploaded pdf filter:\n{pdf_filtered_search_response}\n"
# )


# print("Performing RAG...")
# # Perform a search on this file
# pdf_filtered_search_response = client.rag_completion(
#     "Are there any test documents?",
#     5,
#     filters={"document_id": generate_id_from_label("pdf 1")},
# )
# print(
#     f"Search response w/ uploaded pdf filter:\n{pdf_filtered_search_response}\n"
# )

# print("Performing RAG with streaming...")


# # Perform a RAG completion with streaming
# async def stream_rag_completion():
#     async for chunk in client.stream_rag_completion(
#         "Are there any test documents?",
#         5,
#         filters={"document_id": generate_id_from_label("pdf 1")},
#         generation_config={"stream": True},
#     ):
#         print(chunk, end="", flush=True)


# asyncio.run(stream_rag_completion())

# print("Fetching logs after all steps...")
# logs_response = client.get_logs()
# print(f"Logs response:\n{logs_response}\n")

# print("Fetching logs summary after all steps...")
# logs_summary_response = client.get_logs_summary()
# print(f"Logs summary response:\n{logs_summary_response}\n")



import os
from typing import Union

from r2r.core.providers.eval import EvalProvider


class PareaEvalProvider(EvalProvider):
    def __init__(self, provider: str, sampling_fraction: float = 1.0):
        super().__init__(provider, sampling_fraction)
        try:
            from parea.evals.general import answer_relevancy_factory
            from parea.evals.rag import (
                answer_context_faithfulness_statement_level_factory,
                context_query_relevancy_factory,
                context_ranking_pointwise_factory,
            )
            from parea.schemas.log import Log

            self.answer_relevancy = answer_relevancy_factory()
            self.context_query_relevancy = context_query_relevancy_factory()
            self.context_ranking_pointwise = (
                context_ranking_pointwise_factory()
            )
            self.answer_context_faithfulness_statement_level = (
                answer_context_faithfulness_statement_level_factory()
            )

            def create_log(query: str, context: str, completion: str) -> Log:
                return Log(
                    inputs={
                        "question": query,
                        "context": context,
                    },
                    output=completion,
                )

            self._create_log = create_log
        except ImportError:
            raise ImportError(
                "Parea is not installed. Please install it using `pip install parea-ai`."
            )
        if not os.getenv("OPENAI_API_KEY"):
            raise ValueError(
                "Please set the `OPENAI_API_KEY` environment variable to run with Parea."
            )

    def _evaluate(
        self, query: str, context: str, completion: str
    ) -> dict[str, dict[str, Union[str, float]]]:
        log = self._create_log(query, context, completion)

        answer_relevancy_score = self.answer_relevancy(log)
        context_query_relevancy_score = self.context_query_relevancy(log)
        context_ranking_pointwise_score = self.context_ranking_pointwise(log)
        answer_context_faithfulness_statement_level_score = (
            self.answer_context_faithfulness_statement_level(log)
        )

        return {
            "context_query_relevancy": {
                "score": context_query_relevancy_score,
            },
            "context_ranking": {
                "score": context_ranking_pointwise_score,
            },
            "answer_relevancy": {
                "score": answer_relevancy_score,
            },
            "answer_context_faithfulness": {
                "score": answer_context_faithfulness_statement_level_score,
            },
        }
